@@ -93,14 +93,15 @@ snd_pcm_t* initAlsa(char *dev, int format, int channels, int speed, int mode)
 	snd_pcm_t *handle;
 	snd_pcm_hw_params_t *hwparams;
 	int err;
-	int periods;
+	unsigned int periods;
   
 #ifdef DRIVER_ALSA_09_NEW_PCM_API
 	int t_dir=0;
-	int t_speed=speed;
+	unsigned int t_speed=(unsigned int)speed;
 	snd_pcm_uframes_t t_bufsize=BUFFERSIZE;
 #endif
 
+	snd_config_update_free_global();
 	err = snd_pcm_open(&handle, dev, mode, SND_PCM_NONBLOCK);
 	if (err < 0) {
 		if (alsadbg)
@@ -389,10 +390,14 @@ void esd_audio_close(void)
 		print_state();
 	}
 
-	if (alsa_playback_handle != NULL)
+	if (alsa_playback_handle != NULL) {
+		snd_pcm_hw_free( alsa_playback_handle );
 		snd_pcm_close( alsa_playback_handle );
-	if (alsa_capture_handle != NULL)
+	}
+	if (alsa_capture_handle != NULL) {
+		snd_pcm_hw_free( alsa_capture_handle );
 		snd_pcm_close(alsa_capture_handle);
+	}
 	alsa_playback_handle = NULL;
 	alsa_capture_handle = NULL;
 }
