@@ -22,6 +22,11 @@
 #define gethostbyname2(host, family) gethostbyname((host))
 #endif /* HAVE_GETHOSTBYNAME2 */
 
+#ifndef SUN_LEN
+#define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path)  \
+                     + strlen ((ptr)->sun_path))
+#endif
+
 /* AIX defines this */
 #ifndef h_errno
 extern int h_errno;
@@ -30,6 +35,34 @@ extern int h_errno;
 /* max arguments (argc + tokenized esd.conf) can't be more than this */
 #define MAX_OPTS 128
 
+typedef const char *(*audio_devices_t)(void);
+typedef int (*audio_open_t)(void);
+typedef void (*audio_close_t)(void);
+typedef void (*audio_pause_t)(void);
+typedef void (*audio_flush_t)(void);
+typedef int (*audio_write_t)( void *buffer, int buf_size );
+typedef int (*audio_read_t)( void *buffer, int buf_size );
+
+audio_devices_t impl_esd_audio_devices = esd_audio_devices;
+audio_open_t impl_esd_audio_open = esd_audio_open;
+audio_close_t impl_esd_audio_close = esd_audio_close;
+audio_pause_t impl_esd_audio_pause = esd_audio_pause;
+audio_flush_t impl_esd_audio_flush = esd_audio_flush;
+audio_write_t impl_esd_audio_write = esd_audio_write;
+audio_read_t impl_esd_audio_read = esd_audio_read;
+
+#define esd_audio_devices impl_esd_audio_devices
+#define esd_audio_open impl_esd_audio_open
+#define esd_audio_close impl_esd_audio_close
+#define esd_audio_pause impl_esd_audio_pause
+#define esd_audio_flush impl_esd_audio_flush
+#define esd_audio_write impl_esd_audio_write
+#define esd_audio_read impl_esd_audio_read
+
+#ifdef DRIVER_ARTS
+#include "audio_arts.c"
+#endif
+
 /*******************************************************************/
 /* esd.c - prototypes */
 void set_audio_buffer( void *buf, esd_format_t format, int magl, int magr,
@@ -37,6 +70,7 @@ void set_audio_buffer( void *buf, esd_format_t format, int magl, int magr,
 void clean_exit( int signum );
 void reset_signal( int signum );
 void reset_daemon( int signum );
+void reconnect_driver( int signum );
 int open_listen_socket( const char *hostname, int port );
 
 /* from esd_config.c */
@@ -49,6 +83,8 @@ int esd_is_owned = 0;		 /* start without owner, first client claims it */
 int esd_is_locked = 1;		 /* if owned, will prohibit foreign sources */
 char esd_owner_key[ESD_KEY_LEN]; /* the key that locks the daemon */
 
+int esd_pending_driver_reconnect = 0; /* set by SIGUSR1 handler for
+                                         reconnecting to the audio card */
 int esd_on_standby = 0;		/* set to ignore incoming audio data */
 int esdbg_trace = 0;		/* show warm fuzzy debug messages */
 int esdbg_comms = 0;		/* show protocol level debug messages */
@@ -70,7 +106,7 @@ int esd_use_tcpip = 0;          /* use tcp/ip sockets instead of unix domain */
 int esd_terminate = 0;          /* terminate after the last client exits */
 int esd_public = 0;             /* allow connects from hosts other than localhost */
 int esd_spawnpid = 0;           /* The PID of the process that spawned us (for use by esdlib only) */
-int esd_spawnfd = 0;            /* The FD of the process that spawned us (for use by esdlib only) */
+int esd_spawnfd = -1;           /* The FD of the process that spawned us (for use by esdlib only) */
 
 #if defined ENABLE_IPV6        
 int esd_use_ipv6 = 0;          /* We need it in accept () to know if we use 
@@ -191,6 +227,15 @@ void clean_exit(int signum) {
     exit( 0 );
 }
 
+void reconnect_driver( int signum )
+{
+    ESDBG_TRACE(
+       printf( "(ca) reconnecting to driver on signal %d\n",
+               signum ); );
+    esd_pending_driver_reconnect = 1;
+    signal( signum, reconnect_driver );
+}
+
 void reset_signal(int signum) {
 /*    fprintf( stderr, "received signal %d: resetting...\n", signum );*/
     signal( signum, reset_signal);
@@ -210,14 +255,20 @@ esd_connect_unix(void)
   if ( socket_out < 0 )
     return -1;
   /* this was borrowed blindly from the Tcl socket stuff */
-  if ( fcntl( socket_out, F_SETFD, FD_CLOEXEC ) < 0 )
+  if ( fcntl( socket_out, F_SETFD, FD_CLOEXEC ) < 0 ) {
+    close (socket_out);
     return -1;
+  }
   if ( setsockopt( socket_out, SOL_SOCKET, SO_REUSEADDR,
-		  &curstate, sizeof(curstate) ) < 0 )
+		  &curstate, sizeof(curstate) ) < 0 ) {
+    close (socket_out);
     return -1;
+  }
+
   /* set the connect information */
   socket_unix.sun_family = AF_UNIX;
-  strncpy(socket_unix.sun_path, ESD_UNIX_SOCKET_NAME, sizeof(socket_unix.sun_path));
+  strncpy(socket_unix.sun_path, ESD_UNIX_SOCKET_NAME, sizeof(socket_unix.sun_path)-1);
+  socket_unix.sun_path[sizeof(socket_unix.sun_path)-1] = '\0';
   if ( connect( socket_out,
 	       (struct sockaddr *) &socket_unix, SUN_LEN(&socket_unix) ) < 0 )
     return -1;
@@ -341,10 +392,10 @@ int open_listen_socket(const char *hostname, int port )
     struct sockaddr_un socket_unix;
     int socket_listen = -1;
     struct linger lin;
-    size_t socket_len;
-    struct sockaddr *saddr;
+    size_t socket_len = 0;
+    struct sockaddr *saddr = NULL;
 
-	struct hostent *resolved;
+    struct hostent *resolved;
 
 
     /* create the socket, and set for non-blocking */
@@ -410,7 +461,6 @@ int open_listen_socket(const char *hostname, int port )
         {
 		if (!(resolved=gethostbyname2(hostname, AF_INET)))
                 {
-			hstrerror(h_errno);
 			return -1;
                 }
                 memcpy(&(socket_addr.sin_addr), resolved->h_addr_list[0], resolved->h_length);
@@ -574,15 +624,18 @@ int main ( int argc, char *argv[] )
     /* Enlightened sound Daemon */
 
     int esd_port = ESD_DEFAULT_PORT;
-    int length = 0;
     int arg = 0;
     int itmp;
 
     void *output_buffer = NULL;
+    extern signed int *mixed_buffer;
 
     char *hostname=NULL;
     char *endptr;
 
+    int fd0;
+    int max_fds;
+
     /* from esd_config.c */
     extern char esd_spawn_options[];
     extern char esd_default_options[];
@@ -599,17 +652,19 @@ int main ( int argc, char *argv[] )
     /* begin test scaffolding parameters */
     /* int format = AFMT_U8; AFMT_S16_LE; */
     /* int stereo = 0; */     /* 0=mono, 1=stereo */
-    int default_rate = ESD_DEFAULT_RATE, default_buf_size = ESD_BUF_SIZE;
+    int default_rate = ESD_DEFAULT_RATE;
     int i, j, freq=440;
     int magl, magr;
 
-    int first = 1;
-
     int default_format = ESD_BITS16 | ESD_STEREO;
     /* end test scaffolding parameters */
 
     programname = *argv;
 
+#ifdef DRIVER_ARTS
+    artschk();
+#endif
+
     /* read in esd.conf, ~/.esd.conf, ESD_SPAWN_OPTIONS or ESD_OPTIONS*/
     esd_config_read();
     
@@ -700,7 +755,7 @@ int main ( int argc, char *argv[] )
 	} else if ( !strcmp( opts[ arg ], "-as" ) ) {
 	    if ( ++arg != num_opts ) {
 		esd_autostandby_secs = strtol ( opts[arg], &endptr, 10);
-		if ( !esd_autostandby_secs && !endptr && !*endptr) {
+		if ( !esd_autostandby_secs && (!endptr || !*endptr) ) {
 		    esd_autostandby_secs = ESD_DEFAULT_AUTOSTANDBY_SECS;
 		    fprintf( stderr, "- could not read autostandby timeout: %s\n",
 			     opts[ arg ] );
@@ -741,7 +796,7 @@ int main ( int argc, char *argv[] )
 		esd_spawnpid = atoi( opts[ arg ] );
 	} else if ( !strcmp( opts[ arg ], "-spawnfd" ) ) {
 	    if ( ++arg < num_opts )
-		esd_spawnfd = atoi( opts[ arg ] );
+		sscanf ( opts [ arg ], "%i", &esd_spawnfd);
 	} else if ( !strcmp( opts[ arg ], "-trust" ) ) {
 	    esd_trustval = 0;
 	} else if ( !strcmp( opts[ arg ], "-v" ) || !strcmp( opts[ arg ], "--version" ) ) {
@@ -779,6 +834,18 @@ int main ( int argc, char *argv[] )
 	}
     }
 
+    /* cd to / */
+    chdir ("/");
+
+    /* close all open file descriptors */
+    max_fds = getdtablesize();
+    for(i=0;i<max_fds;i++){
+      close(i);
+    }
+    fd0 = open("/dev/null", O_RDWR);
+    dup(0);
+    dup(0);
+
     /* open the listening socket */
   listen_socket = open_listen_socket(hostname, esd_port );
   if ( listen_socket < 0 ) {
@@ -791,11 +858,6 @@ int main ( int argc, char *argv[] )
     exit( 1 );	
   }
 
-#define ESD_AUDIO_STUFF \
-    esd_sample_size = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 ) \
-	? sizeof(signed short) : sizeof(unsigned char); \
-    esd_buf_size_samples = default_buf_size / 2; \
-    esd_buf_size_octets = esd_buf_size_samples * esd_sample_size;
 
     /* start the initializatin process */
 /*    printf( "ESound daemon initializing...\n" );*/
@@ -803,7 +865,6 @@ int main ( int argc, char *argv[] )
     /* set the data size parameters */
     esd_audio_format = default_format;
     esd_audio_rate = default_rate;
-    ESD_AUDIO_STUFF;
 
   /* open and initialize the audio device, /dev/dsp */
   itmp = esd_audio_open();
@@ -812,7 +873,7 @@ int main ( int argc, char *argv[] )
     if(esd_spawnpid)
       kill(esd_spawnpid, SIGALRM); /* Startup failed */
     
-    if(esd_spawnfd) {
+    if(esd_spawnfd >= 0) {
 	char c = 0; /* Startup failed */
 	write (esd_spawnfd, &c, 1);
     }
@@ -828,7 +889,6 @@ int main ( int argc, char *argv[] )
     /* cant do defaults ... try 44.1 kkz 8bit stereo */
     esd_audio_format = ESD_BITS8 | ESD_STEREO;
     esd_audio_rate = 44100;
-    ESD_AUDIO_STUFF;
     if ( esd_audio_open() < 0 ) {
       fprintf(stderr, "Audio device open for 44.1Khz, stereo, 8bit failed\n"
 
@@ -836,56 +896,48 @@ int main ( int argc, char *argv[] )
     /* cant do defaults ... try 48 kkz 16bit stereo */
     esd_audio_format = ESD_BITS16 | ESD_STEREO;
     esd_audio_rate = 48000;
-    ESD_AUDIO_STUFF;
     if ( esd_audio_open() < 0 ) {
       fprintf(stderr, "Audio device open for 48Khz, stereo,16bit failed\n"
 	      "Trying 22.05Khz, 8bit stereo.\n");
       /* cant do defaults ... try 22.05 kkz 8bit stereo */
       esd_audio_format = ESD_BITS8 | ESD_STEREO;
       esd_audio_rate = 22050;
-      ESD_AUDIO_STUFF;
       if ( esd_audio_open() < 0 ) {
 	fprintf(stderr, "Audio device open for 22.05Khz, stereo, 8bit failed\n"
 		"Trying 44.1Khz, 16bit mono.\n");
 	/* cant do defaults ... try 44.1Khz kkz 16bit mono */
 	esd_audio_format = ESD_BITS16;
 	esd_audio_rate = 44100;
-	ESD_AUDIO_STUFF;
 	if ( esd_audio_open() < 0 ) {
 	  fprintf(stderr, "Audio device open for 44.1Khz, mono, 8bit failed\n"
 		  "Trying 22.05Khz, 8bit mono.\n");
 	  /* cant do defaults ... try 22.05 kkz 8bit mono */
 	  esd_audio_format = ESD_BITS8;
 	  esd_audio_rate = 22050;
-	  ESD_AUDIO_STUFF;
 	  if ( esd_audio_open() < 0 ) {
 	    fprintf(stderr, "Audio device open for 22.05Khz, mono, 8bit failed\n"
 		    "Trying 11.025Khz, 8bit stereo.\n");
 	    /* cant to defaults ... try 11.025 kkz 8bit stereo */
 	    esd_audio_format = ESD_BITS8 | ESD_STEREO;
 	    esd_audio_rate = 11025;
-	    ESD_AUDIO_STUFF;
 	    if ( esd_audio_open() < 0 ) {
 	      fprintf(stderr, "Audio device open for 11.025Khz, stereo, 8bit failed\n"
 		      "Trying 11.025Khz, 8bit mono.\n");
 	      /* cant to defaults ... try 11.025 kkz 8bit mono */
 	      esd_audio_format = ESD_BITS8;
 	      esd_audio_rate = 11025;
-	      ESD_AUDIO_STUFF;
 	      if ( esd_audio_open() < 0 ) {
 		fprintf(stderr, "Audio device open for 11.025Khz, mono, 8bit failed\n"
 			"Trying 8.192Khz, 8bit mono.\n");
 	        /* cant to defaults ... try 8.192 kkz 8bit mono */
 		esd_audio_format = ESD_BITS8;
 		esd_audio_rate = 8192;
-		ESD_AUDIO_STUFF;
 		if ( esd_audio_open() < 0 ) {
 		  fprintf(stderr, "Audio device open for 8.192Khz, mono, 8bit failed\n"
 			  "Trying 8Khz, 8bit mono.\n");
 		  /* cant to defaults ... try 8 kkz 8bit mono */
 		  esd_audio_format = ESD_BITS8;
 		  esd_audio_rate = 8000;
-		  ESD_AUDIO_STUFF;
 		  if ( esd_audio_open() < 0 ) {
 		    fprintf(stderr, "Sound device inadequate for Esound. Fatal.\n");
 		    if (!esd_use_tcpip)
@@ -896,7 +948,7 @@ int main ( int argc, char *argv[] )
 		    if(esd_spawnpid)
 		      kill(esd_spawnpid, SIGALRM); /* Startup failed */
 
-		    if(esd_spawnfd) {
+		    if(esd_spawnfd >= 0) {
 			char c = 0; /* Startup failed */
 			write (esd_spawnfd, &c, 1);
 		    }
@@ -912,9 +964,15 @@ int main ( int argc, char *argv[] )
     }
    }
   }
+    esd_sample_size = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 )
+	? sizeof(signed short) : sizeof(unsigned char);
+    esd_buf_size_samples = esound_getblksize() / esd_sample_size;
+    esd_buf_size_octets = esound_getblksize();
+
 
     /* allocate and zero out buffer */
     output_buffer = (void *) malloc( esd_buf_size_octets );
+    mixed_buffer = malloc( esd_buf_size_samples * sizeof(*mixed_buffer) );
     memset( output_buffer, 0, esd_buf_size_octets );
 
 
@@ -923,10 +981,11 @@ int main ( int argc, char *argv[] )
     signal( SIGTERM, clean_exit );	/* for default kill */
     signal( SIGPIPE, reset_signal );	/* for closed rec/mon clients */
     signal( SIGHUP, reset_daemon );	/* kill -HUP clear ownership */
+    signal( SIGUSR1, reconnect_driver ); /* react to changed sound card */
 
     /* send some sine waves just to check the sound connection */
     i = 0;
-    if ( esd_beeps ) {
+    if ( esd_spawnfd >= 0 && esd_beeps ) {
 	magl = magr = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16)
 	    ? 30000 : 100;
 
@@ -940,121 +999,21 @@ int main ( int argc, char *argv[] )
 		esd_audio_write( output_buffer, esd_buf_size_octets );
 	    }
 	}
+	esd_audio_pause();
     }
 
-    /* put some stuff in the sound driver before pausing */
-    esd_audio_write( NULL, 0);
-
-    /* pause the sound output */
-    esd_audio_pause();
-
     /* Startup succeeded */
     if(esd_spawnpid)
       kill(esd_spawnpid, SIGUSR1);
 
-    if(esd_spawnfd) {
+    if(esd_spawnfd >= 0) {
 	char c = 1; /* Startup succeeded */
 	write (esd_spawnfd, &c, 1);
+	close (esd_spawnfd);
     }
 
-    /* until we kill the daemon */
-    while ( 1 )
-    {
-	/* block while waiting for more clients and new data */
-	wait_for_clients_and_data( listen_socket );
-
-	/* accept new connections */
-	get_new_clients( listen_socket );
-
-
-	if ((esd_clients_list == NULL) && (!first) && (esd_terminate)) {
-/*	  fprintf(stderr, "No clients!\n");*/
-	  clean_exit(0);
-	  exit(0);
-	}
-
-	/* check for new protocol requests */
-	poll_client_requests();
-	first = 0;
-
-	/* mix new requests, and output to device */
-	refresh_mix_funcs(); /* TODO: set a flag to cue when to do this */
-	length = mix_players( output_buffer, esd_buf_size_octets );
-	
-	/* awaken if on autostandby and doing anything */
-	if ( esd_on_autostandby && length && !esd_forced_standby ) {
-	    ESDBG_TRACE( printf( "stuff to play, waking up.\n" ); );
-	    esd_server_resume();
-	}
-
-	/* we handle this even when length == 0 because a filter could have
-	 * closed, and we don't want to eat the processor if one did.. */
-	if ( esd_filter_list && !esd_on_standby ) {
-	    length = filter_write( output_buffer, length,
-				   esd_audio_format, esd_audio_rate );
-	}
-	
-	if ( length > 0 /* || esd_monitor */ ) {
-	    /* do_sleep = 0; */
-	    if ( !esd_on_standby ) {
-		/* standby check goes in here, so esd will eat sound data */
-		/* TODO: eat a round of data with a better algorithm */
-		/*        this will cause guaranteed timing issues */
-		/* TODO: on monitor, why isn't this a buffer of zeroes? */
-		/* esd_audio_write( output_buffer, esd_buf_size_octets ); */
-		esd_audio_write( output_buffer, length );
-		/* esd_audio_flush(); */ /* this is overkill */
-		esd_last_activity = time( NULL );
-	    }
-	} else {
-	    /* should be pausing just fine within wait_for_clients_and_data */
-	    /* if so, this isn't really needed */
-
-	    /* be very quiet, and wait for a wabbit to come along */
-#if 0
-	    if ( !do_sleep ) { ESDBG_TRACE( printf( "pausing in esd.c\n" ); ); }
-	    do_sleep = 1;
-	    esd_audio_pause();
-#endif
-	}
-
-	/* if someone's monitoring the sound stream, send them data */
-	/* mix_players, above, forces buffer to zero if no players */
-	/* this clears out any leftovers from recording, below */
-	if ( esd_monitor_list && !esd_on_standby && length ) {
-	/* if ( esd_monitor_list && !esd_on_standby ) {  */
-	    monitor_write( output_buffer, length );
-	}
-
-	/* if someone's recording the sound stream, send them data */
-	if ( esd_recorder_list && !esd_on_standby ) {
-	    length = esd_audio_read( output_buffer, esd_buf_size_octets );
-	    if ( length ) {
-		length = recorder_write( output_buffer, length );
-		esd_last_activity = time( NULL );
-	    }
-	}
-
-	if ( esd_on_standby ) {
-#ifdef HAVE_NANOSLEEP
-	    struct timespec restrain;
-	    restrain.tv_sec = 0;
-	    /* funky math to make sure a long can hold it all, calulate in ms */
-	    restrain.tv_nsec = (long) esd_buf_size_samples * 1000L
-		/ (long) esd_audio_rate / 4L;   /* divide by two for stereo */
-	    restrain.tv_nsec *= 1000000L;       /* convert to nanoseconds */
-	    nanosleep( &restrain, NULL );
-#else
-	    struct timeval restrain;
-	    restrain.tv_sec = 0;
-	    /* funky math to make sure a long can hold it all, calulate in ms */
-	    restrain.tv_usec = (long) esd_buf_size_samples * 1000L
-		/ (long) esd_audio_rate / 4L; 	/* divide by two for stereo */
-	    restrain.tv_usec *= 1000L; 		/* convert to microseconds */
-	    select( 0, 0, 0, 0, &restrain );
-#endif
-	}
-    } /* while ( 1 ) */
+    esd_comm_loop(listen_socket,output_buffer,esd_terminate);
+    close (listen_socket);
 
     /* how we'd get here, i have no idea, should only exit on signal */
     clean_exit( -1 );
