@@ -25,10 +25,12 @@ int main(int argc, char **argv)
     {
 	if (!strcmp("-h",argv[arg]))
 	{
-	    printf("usage:\n\t%s [-s server] [-b] [-m] [-r freq] < file\n",
+	    printf("usage:\n\t%s [-s server] [-n name] [-b] [-m] [-r freq] < file\n",
 		   argv[0]);
 	    exit(0);
 	}
+	else if ( !strcmp( "-n", argv[ arg ] ) )
+	    name = argv[ ++arg ];
 	else if ( !strcmp( "-s", argv[ arg ] ) )
 	    host = argv[ ++arg ];
 	else if ( !strcmp( "-b", argv[ arg ] ) )
@@ -61,7 +63,7 @@ int main(int argc, char **argv)
 #endif
 
     format = bits | channels | mode | func;
-    printf( "opening socket, format = 0x%08x at %d Hz\n", 
+    fprintf( stderr, "opening socket, format = 0x%08x at %d Hz\n", 
 	    format, rate );
    
     /* sock = esd_play_stream( format, rate ); */
@@ -79,3 +81,4 @@ int main(int argc, char **argv)
 
     return 0;
 }
+
