@@ -30,6 +30,7 @@ void exit_usage( const char *who, int errcode, const char *why, const char *what
 	     "\n"
 	     "options:\n"
 	     "-s, --server=host:port        contact esd server on host at port\n"
+	     "-p, --prefix=string	    prefix for cached samples\n"
 	     "\n"
 	     "commands:\n"
 	     "lock                          foreign clients may not use the server\n"
@@ -38,7 +39,7 @@ void exit_usage( const char *who, int errcode, const char *why, const char *what
 	     "resume, on                    resume sound output\n"
 	     "cache sample                  cache a sample in the server\n"
 	     "getid name                    retrieve a sample id from its name\n"
-	     "free name                     uncache a sample in the server\n"
+	     "free name                     remove a sample from the cache in the server\n"
 	     "play name                     play a cached sample once\n"
 	     "loop name                     make a cached sample loop\n"
 	     "stop name                     stop the looping sample at end\n"
@@ -58,17 +59,18 @@ int main(int argc, char **argv)
 {
     int esd = -1, arg = 0, option_index = 0;
     int id = 0, left = 256, right = 256, ok;
-    char *server = NULL;
+    char *server = NULL, *prefix = "esound";
     esd_server_info_t *server_info = NULL;
     esd_info_t *all_info = NULL;
     int mode = 0;
   
     struct option opts[] = {
 	{ "server", required_argument, NULL, 's' },
+	{ "prefix", required_argument, NULL, 'p' },
 	{ "file", required_argument, NULL, 'f' },
 	{ "help", no_argument, NULL, 'h' },
 	{ "version", no_argument, NULL, 'v' },
-	{ 0, 0, 0, 0 }
+	{ NULL, 0, NULL, 0 }
     };
 
     /* quick sanity check, check number of args */
@@ -78,7 +80,7 @@ int main(int argc, char **argv)
     /* check the command line areguments */
     do
     {
-	arg = getopt_long(argc, argv, "s:hv", opts, &option_index);
+	arg = getopt_long(argc, argv, "s:p:hv", opts, &option_index);
 
 	switch (arg)
 	{
@@ -86,6 +88,10 @@ int main(int argc, char **argv)
 	    server = strdup( optarg );
 	    break;
 
+	case 'p':
+	    prefix = strdup(optarg);
+	    break;
+
 	case 'h':
 	    exit_usage( argv[ 0 ], 0, NULL, NULL );
 	    break;
@@ -137,6 +143,10 @@ int main(int argc, char **argv)
 	else if ( !strcmp( "resume", argv[ optind ] ) ) {
 	    esd_resume( esd );
 	}
+	else if ( !strcmp( "cache", argv[ optind ] ) ) {
+	    ok = esd_file_cache( esd, argv[ 0 ], argv[ ++optind ] );
+	    printf( "Sample <%d> uploaded. Name = %s:%s\n", ok, argv[ 0 ], argv[ optind ] );
+	}
 	else if ( !strcmp( "getid", argv[ optind ] ) ) {
 	    ok = esd_sample_getid( esd, argv[ ++optind ] );
 	    printf( "%d\n", ok );
@@ -146,9 +156,17 @@ int main(int argc, char **argv)
 	    printf( "%d\n", ok );
 	}
 	else if ( !strcmp( "play", argv[ optind ] ) ) {
-	    ok = esd_sample_free( esd, esd_sample_getid(esd, argv[ ++optind ]) );
+	    ok = esd_sample_play( esd, esd_sample_getid(esd, argv[ ++optind ]) );
 	    printf( "%d\n", ok );
 	}
+        else if ( !strcmp( "loop", argv[ optind ] ) ) {
+           ok = esd_sample_loop( esd, esd_sample_getid(esd, argv[ ++optind ]) );
+           printf( "%d\n", ok );
+        }
+        else if ( !strcmp( "stop", argv[ optind ] ) ) {
+           ok = esd_sample_stop( esd, esd_sample_getid(esd, argv[ ++optind ]) );
+           printf( "%d\n", ok );
+        }
 	else if ( !strcmp( "serverinfo", argv[ optind ] ) ) {
 	    server_info = esd_get_server_info( esd );
 
@@ -199,7 +217,7 @@ int main(int argc, char **argv)
 	    switch( mode )
 	    {
 	    case ESM_ERROR:
-		printf( "standbymode failed\n" );
+		printf( "standby mode failed\n" );
 		break;
 	    case ESM_ON_STANDBY:
 		printf( "server is on standby\n" );
@@ -211,7 +229,7 @@ int main(int argc, char **argv)
 		printf( "server is running\n" );
 		break;
 	    default:
-		printf( "standbymode failed - unexpected return value\n" );
+		printf( "standby mode failed - unexpected return value\n" );
 		break;
 	    };
 	}
@@ -225,3 +243,4 @@ int main(int argc, char **argv)
     close( esd );
     return 0;
 }
+
