@@ -825,15 +825,22 @@ static int is_host_local(const char* host)
  * to ESD.
  */
 #define min(a,b) ( ( (a)<(b) ) ? (a) : (b) )
-int esd_open_sound( const char *host )
+int esd_open_sound( const char *rhost )
 {
     int socket_out = -1;
     int len;
     char use_unix = 0;
     char display_host[ 256 ];
     const char *display;
+    char *host = NULL;
 
-    if ( !host ) host = getenv("ESPEAKER");
+    if ( rhost )
+	host = strdup(rhost);
+    else {
+	char *espeaker = getenv("ESPEAKER");
+	if ( espeaker )
+	    host = strdup(espeaker);
+    }
 
     display = getenv( "DISPLAY" );
     if ( !(host && *host) && display ) {
@@ -845,7 +852,8 @@ int esd_open_sound( const char *host )
 	    len = min( len, 255 ); 
 	    strncpy( display_host, display, len );
 	    display_host[ len ] = '\0';
-	    host = display_host;
+	    if ( host ) free(host);
+	    host = strdup(display_host);
 	}
     }
 
@@ -882,21 +890,51 @@ int esd_open_sound( const char *host )
 	/* there's something inherently flaky about this, and if
 	   there's no audio device, Bad Things Happen */
 
+        /* do not bother trying if esd does not exist */
+        if (access(SERVERDIR"/esd", X_OK) != 0)
+          goto finish_connect;
+
 	if (pipe (esd_pipe) < 0)
 	  goto finish_connect;
 
 	childpid = fork();
 	if(!childpid) {
+	    char *preload2;
+	    char *lib;
+	    char *cmd;
+
 	    /* child process */
+	    close (esd_pipe[0]);
+	    /*
+	     * pull libesddsp out of LD_PRELOAD if it is there
+	     */
+	    preload2 = getenv("LD_PRELOAD");
+	    if (preload2)
+		while ((lib = strstr(preload2,"libesddsp"))) {
+		    char *preload = preload2;
+		    char *start = preload;
+		    char *end   = preload+strcspn(preload," \t\n\0");
+		    int length;
+
+		    while(end < lib) {
+			start = end+1;
+			end   = start+strcspn(start,"\t\n\0");
+		    }
+		    length = (preload+strlen(preload))-(end+1);
+		    preload2 = malloc(11+(start-preload)+length);
+		    strcpy(preload2, "LD_PRELOAD=");
+		    strncat(preload2, preload, start-preload);
+		    strncat(preload2, end+1, length);
+		    putenv(preload2);
+		}
+	    cmd = malloc(strlen(SERVERDIR"/esd  -spawnfd 9999999999") + strlen(esd_spawn_options));
+	    sprintf(cmd, "%s/esd %s -spawnfd %d", SERVERDIR, esd_spawn_options, esd_pipe[1]);
+
 	    if(!fork()) {
-		/* child of child process */
-		char *cmd;
-
+		/* child of child process.  Minimal so startup overhead is
+		 * not included in the waiting time
+		 */
 		setsid();
-		cmd = malloc(strlen(SERVERDIR"/esd  -spawnfd 999999") + (esd_spawn_options?strlen(esd_spawn_options):0));
-
-		sprintf(cmd, "%s/esd %s -spawnfd %d", SERVERDIR, esd_spawn_options?esd_spawn_options:"", esd_pipe[1]);
-
 		execl("/bin/sh", "/bin/sh", "-c", cmd, NULL);
 		perror("execl");
 		_exit(1);
@@ -907,6 +945,7 @@ int esd_open_sound( const char *host )
 	} else {
 	    int estat;
 
+	    close(esd_pipe[1]);
 	    while ((waitpid (childpid, &estat, 0)== -1) && (errno == EINTR));
 	}
 
@@ -933,8 +972,6 @@ int esd_open_sound( const char *host )
 	}
 	    
 	close (esd_pipe[0]);
-	close (esd_pipe[1]);
-	
     }
 #endif
  finish_connect:
@@ -943,6 +980,7 @@ int esd_open_sound( const char *host )
 	close(socket_out); socket_out = -1;
     }
 
+    if ( host ) free(host);
     return socket_out;
 }
 
