@@ -89,22 +89,23 @@ esd_proto_handler_info_t esd_proto_map[ ESD_PROTO_MAX ] =
 int esd_proto_get_latency( esd_client_t *client )
 {
     int lag, amount, actual;
+    int buf_size = esound_getblksize();
 
     ESDBG_TRACE( printf( "(%02d) proto: get latency\n", client->fd ); );
   
     if (esd_audio_format & ESD_STEREO)
       {
 	if (esd_audio_format & ESD_BITS16)
-	  amount = (44100 * (ESD_BUF_SIZE + 64)) / esd_audio_rate;
+	  amount = (44100 * (buf_size + 64)) / esd_audio_rate;
 	else
-	  amount = (44100 * (ESD_BUF_SIZE + 128)) / esd_audio_rate;
+	  amount = (44100 * (buf_size + 128)) / esd_audio_rate;
       }
     else
       {
 	if (esd_audio_format & ESD_BITS16)
-	  amount = (2 * 44100 * (ESD_BUF_SIZE + 128)) / esd_audio_rate;
+	  amount = (2 * 44100 * (buf_size + 128)) / esd_audio_rate;
 	else
-	  amount = (2 * 44100 * (ESD_BUF_SIZE + 256)) / esd_audio_rate;
+	  amount = (2 * 44100 * (buf_size + 256)) / esd_audio_rate;
       }
     lag = maybe_swap_32( client->swap_byte_order, amount );
 
@@ -264,7 +265,8 @@ int esd_proto_standby( esd_client_t *client )
     ok = esd_validate_source( client, client->proto_data, 1 );
 
     if ( ok ) ok = esd_server_standby();
-    esd_forced_standby = 1;
+    if(!esd_on_autostandby)
+	    esd_forced_standby = 1;
 
     client_ok = maybe_swap_32( client->swap_byte_order, ok );
     ESD_WRITE_INT( client->fd, &client_ok, sizeof(client_ok), 
@@ -311,55 +313,56 @@ int esd_proto_stream_play( esd_client_t *client )
 }
 
 /*******************************************************************/
-/* manage the single recording client, return boolean ok */
+/* manage recorder clients, return boolean ok */
 int esd_proto_stream_recorder( esd_client_t *client )
 {
+    esd_player_t *recorder = NULL;
+    
     /* wake up if we're asleep */
     if ( esd_on_autostandby  && !esd_forced_standby ) {
 	ESDBG_TRACE( printf( "stuff to record, waking up.\n" ); );
 	esd_server_resume();
     }
 
-    /* if we're already recording, or (still) in standby mode, go away */
-    if ( esd_recorder || esd_on_standby ) {
+    /* if we're in standby mode, go away */
+    if ( esd_on_standby )
 	return 0;
-    }
 
     /* sign up the new recorder client */
-    esd_recorder = new_stream_player( client );
-    if ( esd_recorder != NULL ) {
-
+    recorder = new_stream_player( client );
+    /* we got one, right? */
+    if ( !recorder )
+	return 0;
+    
+    if (!(esd_audio_format & ESD_RECORD)) {
 	/* let the device know we want to record */
 	ESDBG_TRACE( printf( "closing audio for a sec...\n" ); );
 	esd_audio_close();
-	sleep(1);
+	usleep(100);
 	esd_audio_format |= ESD_RECORD;
 	ESDBG_TRACE( printf( "reopening audio to record...\n" ); );
 	if (esd_audio_open() < 0) {
             /* Failed to record */
-            free_player( esd_recorder );
-            esd_recorder = NULL;
+            free_player( recorder );
             esd_audio_format &= ~ESD_RECORD;
-            sleep(1);
+            usleep(100);
             /* If we fail here, we have a oops */
             esd_audio_open();
             return 0;
         }
 	ESDBG_TRACE( printf( "reopened?\n" ); );
-
-	/* flesh out the recorder */
-	esd_recorder->parent = client;
-	esd_recorder->translate_func 
-	    = get_translate_func( esd_audio_format, esd_audio_rate,
-				  esd_recorder->format, esd_recorder->rate );
-
-	ESDBG_TRACE( printf ( "(%02d) recording on client\n", client->fd ); );
-
-    } else {
-	/* failed to initialize the recorder, kill its client */
-	return 0;
     }
+    
+    add_recorder(recorder);
+    
+    /* flesh out the recorder */
+    recorder->parent = client;
+    recorder->translate_func 
+	    = get_translate_func( esd_audio_format, esd_audio_rate,
+				  recorder->format, recorder->rate );
 
+    ESDBG_TRACE( printf ( "(%02d) recording on client\n", client->fd ); );
+    
     client->state = ESD_STREAMING_DATA;
     return 1;
 }
@@ -834,7 +837,7 @@ int esd_proto_standby_mode( esd_client_t *client )
 
 /*******************************************************************/
 /* checks for new client requiests - returns 1 */
-int poll_client_requests()
+int poll_client_requests(void)
 {
     int can_read, length = 0, is_ok = 0;
     esd_client_t *client = NULL;
